import dataclasses
import queue
import time
from typing import List, Callable

import colorama
from colorama import Fore, Style


@dataclasses.dataclass
class Exercise:
    name: str
    reps: int
    hold_time: float
    rep_delay: float = 2

    def _perform(self) -> None:
        for i in range(self.reps):
            print(
                f"\tNow doing rep: {i + 1}, hold for {self.hold_time} seconds")
            time.sleep(self.hold_time)
            print("\tGet ready to do the next rep!")
            time.sleep(self.rep_delay)
            print(
                f"\tExercise done! Press {Fore.GREEN}ENTER{Style.RESET_ALL} to continue."
            )

    def exec_buf(self) -> List[Callable[[], None]]:
        return [
            lambda: print(
                f"Press {Fore.GREEN}ENTER{Style.RESET_ALL} when ready to "
                f"do exercise {Fore.YELLOW}'{self.name}'{Style.RESET_ALL}"),
            self._perform
        ]


def welcome_msg():
    print(f"Press {Fore.GREEN}ENTER{Style.RESET_ALL} to begin")


def main():
    colorama.init()

    exercises = map(lambda name: Exercise(name, 1, 0.1, 0.1),
                    ["Push up", "Pull down", "Push left", "Push right"])
    exercise_funcs = [
        func for exercise in exercises for func in exercise.exec_buf()
    ]

    exec_queue = queue.SimpleQueue()
    exec_queue.put(welcome_msg)
    for exercise_func in exercise_funcs:
        exec_queue.put(exercise_func)

    while not exec_queue.empty():
        exec_queue.get()()
        input()


if __name__ == "__main__":
    main()


"""setup.py

The setup script of TMJ.

Author:
    Figglewatts <me@figglewatts.co.uk>
"""
from os import path
from setuptools import setup

here = path.join(path.dirname(__file__))


def get_repo_file_content(filename: str) -> str:
    with open(path.join(here, filename), encoding="utf-8") as f:
        return f.read()


setup(
    name="tmj",
    version="v1.0.0",
    description=
    "Small CLI utility script to help me with TMJ disorder isometric jaw exercises.",
    long_description=get_repo_file_content("README.md"),
    long_description_content_type="text/markdown",
    url="https://github.com/Figglewatts/tmj",
    author="Figglewatts",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console", "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.7", "Topic :: Utilities",
        "Typing :: Typed"
    ],
    keywords="tmj temporomandibular joint disorder",
    python_requires=">=3.7",
    install_requires=["colorama>=0.4.3"],
    entry_points={"console_scripts": ["tmj=tmj:main"]},
    py_modules=["tmj"])
